use credentials_example::{
    read_line, DOOR_LISTENER_ADDRESS, DOOR_TCP_ADDRESS, DOOR_WORKER_ADDRESS, OFFICE_ISSUER_ADDRESS,
    OFFICE_LISTENER_ADDRESS, OFFICE_TCP_ADDRESS,
};
use ockam::{
    credential_attribute_values, credential_type, reveal_attributes, route, Context,
    CredentialProtocol, Identity, Identity, IdentityIdentifier, Result, TcpTransport,
    TrustIdentifierPolicy, Vault, TCP,
};

#[ockam::node]
async fn main(ctx: Context) -> Result<()> {
    let vault = Vault::create(&ctx).await?;
    let mut entity = Identity::create(&ctx, &vault).await?;
    println!("Bob id: {}", entity.identifier().await?);

    println!("Enter Office id: ");
    let office_id = read_line();
    let office_id = IdentityIdentifier::try_from(office_id.as_str())?;

    println!("Enter Door id: ");
    let door_id = read_line();
    let door_id = IdentityIdentifier::try_from(door_id.as_str())?;

    let tcp = TcpTransport::create(&ctx).await?;
    tcp.connect(OFFICE_TCP_ADDRESS).await?;

    let office_channel = entity
        .create_secure_channel(
            route![(TCP, OFFICE_TCP_ADDRESS), OFFICE_LISTENER_ADDRESS],
            TrustIdentifierPolicy::new(office_id.clone()),
        )
        .await?;

    let credential = entity
        .acquire_credential(
            route![office_channel, OFFICE_ISSUER_ADDRESS],
            &office_id,
            credential_type!["TYPE_ID"; "door_id", (Number, "can_open_door")],
            credential_attribute_values![door_id.clone().to_string(), 1],
        )
        .await?;

    println!("Bob got credential!");

    tcp.connect(DOOR_TCP_ADDRESS).await?;

    let door_channel = entity
        .create_secure_channel(
            route![(TCP, DOOR_TCP_ADDRESS), DOOR_LISTENER_ADDRESS],
            TrustIdentifierPolicy::new(door_id.clone()),
        )
        .await?;

    entity
        .present_credential(
            route![door_channel, DOOR_WORKER_ADDRESS],
            credential,
            reveal_attributes!["door_id", "can_open_door"],
        )
        .await?;

    println!("Bob presented credential!");

    // TODO: Send actual payload

    Ok(())
}
